@@ -277,12 +277,26 @@ class EntryController extends Controller
         return $this->showEntries('untagged', $request, $page);
     }
 
+    /**
+     * Shows entries with annotations for current user.
+     *
+     * @param int $page
+     *
+     * @Route("/annotated/list/{page}", name="annotated", defaults={"page" = "1"})
+     *
+     * @return \Symfony\Component\HttpFoundation\Response
+     */
+    public function showWithAnnotationsEntriesAction(Request $request, $page)
+    {
+        return $this->showEntries('annotated', $request, $page);
+    }
+
     /**
      * Shows random entry depending on the given type.
      *
      * @param string $type
      *
-     * @Route("/{type}/random", name="random_entry", requirements={"type": "unread|starred|archive|untagged|all"})
+     * @Route("/{type}/random", name="random_entry", requirements={"type": "unread|starred|archive|untagged|annotated|all"})
      *
      * @return \Symfony\Component\HttpFoundation\RedirectResponse
      */
@@ -517,6 +531,20 @@ class EntryController extends Controller
         );
     }
 
+    /**
+     * List the entries with the same domain as the current one.
+     *
+     * @param int $page
+     *
+     * @Route("/domain/{id}/{page}", requirements={"id" = ".+"}, defaults={"page" = 1}, name="same_domain")
+     *
+     * @return \Symfony\Component\HttpFoundation\Response
+     */
+    public function getSameDomainEntries(Request $request, $page = 1)
+    {
+        return $this->showEntries('same-domain', $request, $page);
+    }
+
     /**
      * Global method to retrieve entries depending on the given type
      * It returns the response to be send.
@@ -549,10 +577,16 @@ class EntryController extends Controller
                 $qb = $repository->getBuilderForArchiveByUser($this->getUser()->getId());
                 $formOptions['filter_archived'] = true;
                 break;
+            case 'annotated':
+                $qb = $repository->getBuilderForAnnotationsByUser($this->getUser()->getId());
+                break;
             case 'unread':
                 $qb = $repository->getBuilderForUnreadByUser($this->getUser()->getId());
                 $formOptions['filter_unread'] = true;
                 break;
+            case 'same-domain':
+                $qb = $repository->getBuilderForSameDomainByUser($this->getUser()->getId(), $request->get('id'));
+                break;
             case 'all':
                 $qb = $repository->getBuilderForAllByUser($this->getUser()->getId());
                 break;
