@@ -3,6 +3,7 @@
 namespace Tests\Wallabag\CoreBundle\Controller;
 
 use Tests\Wallabag\CoreBundle\WallabagCoreTestCase;
+use Wallabag\AnnotationBundle\Entity\Annotation;
 use Wallabag\CoreBundle\Entity\Config;
 use Wallabag\CoreBundle\Entity\Entry;
 use Wallabag\CoreBundle\Entity\SiteCredential;
@@ -14,6 +15,7 @@ class EntryControllerTest extends WallabagCoreTestCase
     const AN_URL_CONTAINING_AN_ARTICLE_WITH_IMAGE = 'https://www.lemonde.fr/judo/article/2017/11/11/judo-la-decima-de-teddy-riner_5213605_1556020.html';
     public $downloadImagesEnabled = false;
     public $url = 'https://www.lemonde.fr/pixels/article/2019/06/18/ce-qu-il-faut-savoir-sur-le-libra-la-cryptomonnaie-de-facebook_5477887_4408996.html';
+    private $entryDataTestAttribute = '[data-test="entry"]';
 
     /**
      * @after
@@ -100,14 +102,14 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->request('GET', '/');
 
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         // Good URL
         $client->request('GET', '/bookmarklet', ['url' => $this->url]);
         $this->assertSame(302, $client->getResponse()->getStatusCode());
         $client->followRedirect();
         $crawler = $client->request('GET', '/');
-        $this->assertCount(5, $crawler->filter('li.entry'));
+        $this->assertCount(5, $crawler->filter($this->entryDataTestAttribute));
 
         $em = $client->getContainer()
             ->get('doctrine.orm.entity_manager');
@@ -175,6 +177,42 @@ class EntryControllerTest extends WallabagCoreTestCase
         $client->getContainer()->get('craue_config')->set('store_article_headers', 0);
     }
 
+    /**
+     * @group NetworkCalls
+     */
+    public function testPostNewOkWithTaggingRules()
+    {
+        $this->logInAs('empty');
+        $client = $this->getClient();
+
+        $crawler = $client->request('GET', '/new');
+
+        $this->assertSame(200, $client->getResponse()->getStatusCode());
+
+        $form = $crawler->filter('form[name=entry]')->form();
+
+        $data = [
+            'entry[url]' => $this->url,
+        ];
+
+        $client->submit($form, $data);
+
+        $this->assertSame(302, $client->getResponse()->getStatusCode());
+
+        $content = $client->getContainer()
+            ->get('doctrine.orm.entity_manager')
+            ->getRepository('WallabagCoreBundle:Entry')
+            ->findByUrlAndUserId($this->url, $this->getLoggedInUserId());
+
+        $tags = $content->getTagsLabel();
+
+        /*
+         * Without the custom reading speed of `empty` user, it'll be inversed
+         */
+        $this->assertContains('longread', $tags);
+        $this->assertNotContains('shortread', $tags);
+    }
+
     /**
      * @group NetworkCalls
      */
@@ -411,6 +449,16 @@ class EntryControllerTest extends WallabagCoreTestCase
         $this->assertSame(200, $client->getResponse()->getStatusCode());
     }
 
+    public function testWithAnnotations()
+    {
+        $this->logInAs('admin');
+        $client = $this->getClient();
+
+        $crawler = $client->request('GET', '/annotated/list');
+        $this->assertSame(200, $client->getResponse()->getStatusCode());
+        $this->assertCount(2, $crawler->filter('ol.entries > li'));
+    }
+
     public function testRangeException()
     {
         $this->logInAs('admin');
@@ -723,7 +771,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnReadingTimeWithNegativeValue()
@@ -743,7 +791,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         $crawler = $client->submit($form, $data);
 
         // forcing negative value results in no entry displayed
-        $this->assertCount(0, $crawler->filter('li.entry'));
+        $this->assertCount(0, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnReadingTimeOnlyUpper()
@@ -752,7 +800,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         $client = $this->getClient();
 
         $crawler = $client->request('GET', '/all/list');
-        $this->assertCount(5, $crawler->filter('li.entry'));
+        $this->assertCount(5, $crawler->filter($this->entryDataTestAttribute));
 
         $entry = new Entry($this->getLoggedInUser());
         $entry->setUrl($this->url);
@@ -761,7 +809,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         $this->getEntityManager()->flush();
 
         $crawler = $client->request('GET', '/all/list');
-        $this->assertCount(6, $crawler->filter('li.entry'));
+        $this->assertCount(6, $crawler->filter($this->entryDataTestAttribute));
 
         $form = $crawler->filter('button[id=submit-filter]')->form();
 
@@ -771,7 +819,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(5, $crawler->filter('li.entry'));
+        $this->assertCount(5, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnReadingTimeOnlyLower()
@@ -789,7 +837,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(0, $crawler->filter('li.entry'));
+        $this->assertCount(0, $crawler->filter($this->entryDataTestAttribute));
 
         $entry = new Entry($this->getLoggedInUser());
         $entry->setUrl($this->url);
@@ -798,7 +846,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         $this->getEntityManager()->flush();
 
         $crawler = $client->submit($form, $data);
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnUnreadStatus()
@@ -816,7 +864,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         $entry = new Entry($this->getLoggedInUser());
         $entry->setUrl($this->url);
@@ -826,7 +874,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(5, $crawler->filter('li.entry'));
+        $this->assertCount(5, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnCreationDate()
@@ -857,7 +905,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         $data = [
             'entry_filter[createdAt][left_date]' => $today->format('Y-m-d'),
@@ -866,7 +914,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         $data = [
             'entry_filter[createdAt][left_date]' => '1970-01-01',
@@ -875,7 +923,45 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(0, $crawler->filter('li.entry'));
+        $this->assertCount(0, $crawler->filter($this->entryDataTestAttribute));
+    }
+
+    public function testFilterOnAnnotatedStatus()
+    {
+        $this->logInAs('admin');
+        $client = $this->getClient();
+
+        $crawler = $client->request('GET', '/all/list');
+
+        $form = $crawler->filter('button[id=submit-filter]')->form();
+
+        $data = [
+            'entry_filter[isAnnotated]' => true,
+        ];
+
+        $crawler = $client->submit($form, $data);
+
+        $this->assertCount(2, $crawler->filter('ol.entries > li'));
+
+        $entry = new Entry($this->getLoggedInUser());
+        $entry->setUrl($this->url);
+
+        $em = $this->getClient()->getContainer()->get('doctrine.orm.entity_manager');
+        $user = $em
+            ->getRepository('WallabagUserBundle:User')
+            ->findOneByUserName('admin');
+
+        $annotation = new Annotation($user);
+        $annotation->setEntry($entry);
+        $annotation->setText('This is my annotation /o/');
+        $annotation->setQuote('content');
+
+        $this->getEntityManager()->persist($entry);
+        $this->getEntityManager()->flush();
+
+        $crawler = $client->submit($form, $data);
+
+        $this->assertCount(3, $crawler->filter('ol.entries > li'));
     }
 
     public function testPaginationWithFilter()
@@ -919,7 +1005,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         ];
 
         $crawler = $client->submit($form, $data);
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         $crawler = $client->request('GET', '/unread/list');
         $form = $crawler->filter('button[id=submit-filter]')->form();
@@ -928,7 +1014,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         ];
 
         $crawler = $client->submit($form, $data);
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         $form = $crawler->filter('button[id=submit-filter]')->form();
         $data = [
@@ -936,7 +1022,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         ];
 
         $crawler = $client->submit($form, $data);
-        $this->assertCount(0, $crawler->filter('li.entry'));
+        $this->assertCount(0, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnStatus()
@@ -951,14 +1037,14 @@ class EntryControllerTest extends WallabagCoreTestCase
         $form['entry_filter[isUnread]']->untick();
 
         $crawler = $client->submit($form);
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
 
         $form = $crawler->filter('button[id=submit-filter]')->form();
         $form['entry_filter[isArchived]']->untick();
         $form['entry_filter[isStarred]']->tick();
 
         $crawler = $client->submit($form);
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterPreselectedStatus()
@@ -995,7 +1081,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         $form['entry_filter[isPublic]']->tick();
 
         $crawler = $client->submit($form);
-        $this->assertCount(0, $crawler->filter('li.entry'));
+        $this->assertCount(0, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testPreviewPictureFilter()
@@ -1008,7 +1094,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         $form['entry_filter[previewPicture]']->tick();
 
         $crawler = $client->submit($form);
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testFilterOnLanguage()
@@ -1029,7 +1115,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         ];
 
         $crawler = $client->submit($form, $data);
-        $this->assertCount(3, $crawler->filter('li.entry'));
+        $this->assertCount(3, $crawler->filter($this->entryDataTestAttribute));
 
         $form = $crawler->filter('button[id=submit-filter]')->form();
         $data = [
@@ -1037,7 +1123,7 @@ class EntryControllerTest extends WallabagCoreTestCase
         ];
 
         $crawler = $client->submit($form, $data);
-        $this->assertCount(2, $crawler->filter('li.entry'));
+        $this->assertCount(2, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testShareEntryPublicly()
@@ -1240,7 +1326,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
 
         $entry = new Entry($this->getLoggedInUser());
         $entry->setUrl($this->url);
@@ -1263,7 +1349,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(2, $crawler->filter('li.entry'));
+        $this->assertCount(2, $crawler->filter($this->entryDataTestAttribute));
 
         $crawler = $client->request('GET', '/all/list');
         $form = $crawler->filter('button[id=submit-filter]')->form();
@@ -1274,7 +1360,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(8, $crawler->filter('li.entry'));
+        $this->assertCount(8, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function testSearch()
@@ -1298,7 +1384,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(4, $crawler->filter('li.entry'));
+        $this->assertCount(4, $crawler->filter($this->entryDataTestAttribute));
 
         // Search on starred list
         $crawler = $client->request('GET', '/starred/list');
@@ -1317,7 +1403,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
 
         $crawler = $client->request('GET', '/archive/list');
 
@@ -1336,7 +1422,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
         $client->request('GET', '/delete/' . $entry->getId());
 
         // test on list of all articles
@@ -1349,7 +1435,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(0, $crawler->filter('li.entry'));
+        $this->assertCount(0, $crawler->filter($this->entryDataTestAttribute));
 
         // test url search on list of all articles
         $entry = new Entry($this->getLoggedInUser());
@@ -1368,7 +1454,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
 
         // same as previous test but for case-sensitivity
         $crawler = $client->request('GET', '/all/list');
@@ -1380,7 +1466,7 @@ class EntryControllerTest extends WallabagCoreTestCase
 
         $crawler = $client->submit($form, $data);
 
-        $this->assertCount(1, $crawler->filter('li.entry'));
+        $this->assertCount(1, $crawler->filter($this->entryDataTestAttribute));
     }
 
     public function dataForLanguage()
@@ -1590,6 +1676,10 @@ class EntryControllerTest extends WallabagCoreTestCase
         $this->assertSame(302, $client->getResponse()->getStatusCode());
         $this->assertStringContainsString('/view/', $client->getResponse()->getTargetUrl(), 'Untagged random');
 
+        $client->request('GET', '/annotated/random');
+        $this->assertSame(302, $client->getResponse()->getStatusCode());
+        $this->assertStringContainsString('/view/', $client->getResponse()->getTargetUrl(), 'With annotations random');
+
         $client->request('GET', '/all/random');
         $this->assertSame(302, $client->getResponse()->getStatusCode());
         $this->assertStringContainsString('/view/', $client->getResponse()->getTargetUrl(), 'All random');
@@ -1671,4 +1761,14 @@ class EntryControllerTest extends WallabagCoreTestCase
         $client->request('GET', '/delete/' . $entry2->getId());
         $this->assertSame(404, $client->getResponse()->getStatusCode());
     }
+
+    public function testGetSameDomainEntries()
+    {
+        $this->logInAs('admin');
+        $client = $this->getClient();
+
+        $crawler = $client->request('GET', '/domain/1');
+        $this->assertSame(200, $client->getResponse()->getStatusCode());
+        $this->assertCount(4, $crawler->filter('ol.entries > li'));
+    }
 }
